--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library machxo2;
use machxo2.all;

use work.AX_Pack.all;
use work.config.all;

--
ENTITY top IS
   PORT (
        -- 
		-- JTAG in from FTDI
		--
		M_TMS         		: IN std_logic;
		M_TCK         		: IN std_logic;
		M_TDI         		: IN std_logic;
		M_TDO         		: OUT std_logic;
	    --
		-- JTAG out to Module
		--
		C_TMS         		: OUT std_logic;
		C_TCK         		: OUT std_logic;
		C_TDI         		: OUT std_logic;
		C_TDO         		: IN std_logic;
		-- JTAG to FMC
		FMC_TMS        		: OUT std_logic;
		FMC_TCK        		: OUT std_logic;
		FMC_TDI        		: OUT std_logic;
		FMC_TDO        		: IN std_logic;

		-- Pushbuttons
		S1                  : IN std_logic;
		S2                  : IN std_logic;
		
		-- TE0701-03+ only
		-- Mode switches
		CM0                 : IN std_logic;
		CM1                 : IN std_logic;
		
		-- TE70xx control pins, 3.3V IN power domain
		NOSEQ               : INOUT std_logic;
		EN1                 : OUT std_logic;
		RESIN               : OUT std_logic;
		
		MODE                : OUT std_logic; 
		PGOOD               : INOUT std_logic;
		
		-- HDMI	
		SDA                 : INOUT std_logic;
		SCL                 : INOUT std_logic;
	
		
		
		-- 3.3V out, or PGOOD domain, should be driven low not powered?
		MIO10               : INOUT std_logic; --	 	
		MIO11               : INOUT std_logic; --
		MIO12               : INOUT std_logic; --
		MIO13               : INOUT std_logic; --
		MIO14               : INOUT std_logic; -- UART0.RX << BDBUS0
		MIO15               : IN std_logic;    -- UART0.TX >> BDBUS1
		
		-- Module interface signals
		Y0                 : IN std_logic;   -- i2c scl
		Y1                 : OUT std_logic;  -- i2c sda out
		Y2                 : IN std_logic;   -- PHY_LED2
		Y3                 : OUT std_logic;  -- S2, Reset
		Y4                 : IN std_logic;   -- PHY_LED1
		Y5                 : IN std_logic;   --  i2c sda in
		Y6                 : IN std_logic;   -- LED1/RFU
		
		X6                 : IN std_logic;   -- LED2 :( can not use if FMC disabled!
		
--		TESTIN             : IN std_logic;   -- 
		
		
		ADBUS4              : OUT std_logic; 
		ADBUS7              : IN std_logic; 

		ACBUS4              : IN std_logic; 
		ACBUS5              : IN std_logic; 
		
		BDBUS0              : IN std_logic;  -- TX from FTDI
		BDBUS1              : OUT std_logic; -- RX to FTDI
		

		-- misc control and status
		USB_OC              : IN std_logic;
		SD_DETECT           : IN std_logic;
		SD_WP               : IN std_logic;
		SEL_SD              : OUT std_logic;
		
		VID0                : OUT std_logic;
		VID1                : OUT std_logic;
		VID2                : OUT std_logic;
		
		
		FMC_SDA             : INOUT std_logic;
		FMC_SCL             : OUT std_logic;

		EN_FMC              : OUT std_logic; -- Enable 3.3V FMC and adjustable DCDC
		PG_C2M              : OUT std_logic; -- Signal Power good to FMC
		FMC_PRSNT           : IN std_logic;  -- 0 if FMC card is present
		POK_FMC             : IN std_logic;  -- Power good signal from FMC VADJ DCDC

		-- LED's in MAGJACK
		PHY_LED1            : OUT std_logic;
		PHY_LED2            : OUT std_logic;

		-- extra pins in MIO bank1 PMod J2
		PX6                 : INOUT std_logic;
		PX7                 : INOUT std_logic;

	    -- 3.3V IN powered
		LED1                : OUT std_logic;
		LED2                : OUT std_logic;
		LED3                : OUT std_logic;
		LED4                : OUT std_logic;
		
		-- FMC VADJ powered
		LED5                : OUT std_logic;
		LED6                : OUT std_logic;
		LED7                : OUT std_logic;
		LED8                : OUT std_logic;

       -- Reset_n				: in std_logic;

		dummy               : OUT std_logic
      );
END ENTITY top;
--
ARCHITECTURE translated OF top IS

COMPONENT OSCH
	-- synthesis translate_off
	GENERIC (NOM_FREQ: string := "24.18");
	-- synthesis translate_on
	PORT ( STDBY :IN std_logic;
	OSC :OUT std_logic;
	SEDSTDBY :OUT std_logic);
END COMPONENT OSCH;

 attribute NOM_FREQ : string;
 attribute NOM_FREQ of MyInternalOSC : label is "24.18";

component I2C_to_GPIO8
    port (
		sda: inout  std_logic; 
        sclk: in  std_logic;
		
        GPIO_input: in  std_logic_vector(7 downto 0);
        GPIO_output: out  std_logic_vector(7 downto 0)
		);
end component;

	COMPONENT PORESET
	PORT(
		SYSRST : IN std_logic;
		CLK : IN std_logic;          
		RST : OUT std_logic
		);
	END COMPONENT;

	
signal POR_DONE: std_logic;
signal por_cnt: std_logic_vector(31 downto 0);

signal POR_DELAYED: std_logic;

signal USB_JTAG_RST: std_logic;

signal clk: std_logic;
signal cnt: std_logic_vector(31 downto 0);
signal FMC_JTAGEN: std_logic;
signal FMC_POWERED: std_logic;

signal GPIO_input: std_logic_vector(31 downto 0);
signal GPIO_output: std_logic_vector(31 downto 0);

signal osc_clk: std_logic;

-- power ON, drive all output low if no powered!
signal PON: std_logic;
signal POWER_FAIL: std_logic;

signal BOOTMODE: std_logic;

signal MODE0: std_logic;
signal MODE1: std_logic;
signal MODE2: std_logic;
signal MODE3: std_logic;

signal ext_reset_n  : std_logic; 		

BEGIN
	dummy 		<= MIO10 or MIO11 or MIO12 or MIO13 or SD_WP or USB_OC or S2 or POK_FMC or ADBUS7 or PX6 or PX7;

    --
	-- FMC I2C mux, just remap "3 wire split i2c" to to normal I2C
	--
	FMC_SDA   	<= '0' when Y5='0' else 'Z';
	FMC_SCL 	<= Y0 and PON;
	Y1 			<= FMC_SDA;
    --
	--
	--
	Y3 			<= S1; -- user button
	--
	--
	--
	MODE0 <= '1' when CM1='1' and CM0='1' else '0'; -- Fixed 1.8V
	MODE1 <= '1' when CM1='0' and CM0='1' else '0'; -- Fixed 2.5V
	MODE2 <= '1' when CM1='1' and CM0='0' else '0'; -- Fixed 3.3V
	MODE3 <= '1' when CM1='0' and CM0='0' else '0'; -- Use I2C to select voltage


	-- When SD card in Slot force BOOTMODE to SD!
	BOOTMODE <= SD_DETECT;
	
	-- Select MIO mux to SD or Pmod
	-- Select SD card (not pmod J2)
	-- note: SD card in pmod J2 can be used as non bootable SD or eMMC card (not bootable)
	SEL_SD   <= BOOTMODE;	
	MODE 	 <= BOOTMODE;

    -- Pmod MIO bank0 multiplexer
	--
	-- Connect ZYNQ UART to FTDI Channel B UART
	--
	MIO14 <= 'Z' when PON = '0' else BDBUS0;
	BDBUS1 <= MIO15;

    -- 
	-- This is probably not supported by Xilinx/Digilent drivers
	-- 
	USB_JTAG_RST <= ACBUS4 or ACBUS5;
	
	-- Drive RESIN low if button S1 pressed or USB JTAG forced RST (disabled)
	RESIN 	<= S2; -- and USB_JTAG_RST;
	
	ext_reset_n <= S2;

	--PON 	<= S2 and POR_DELAYED;
	PON 	<= POR_DELAYED;
	
	EN1 	<= POR_DELAYED;
	
	FMC_JTAGEN  <= '0'; -- it is better to have only simple JTAG chain for scew and timing issues, the combined chain may not work at 15mhz what is fixed speed by Xilinx drivers
	
	clk <= osc_clk;
	
	--
	-- LED's 1..4 can always be used no matter jumper or power options
	--
	LED1 		<= not NOSEQ; --  --
    LED2 		<= '0';
	LED3 		<= not BDBUS0; -- UART Activity
	LED4 		<= not MIO15; --
	--
	-- LED's 5..8 can be on only if B2B large banks are powered
	--
	LED5 		<= BOOTMODE; 
	LED6 		<= '0'; --PGOOD; 
	LED7 		<= X6; --POR_DELAYED; --FMC_POWERED and POK_FMC and not FMC_PRSNT; -- when FMC is present and powered
	LED8 		<= Y6; --PON; --Y6; 			-- USER LED


    NOSEQ <= 'Z';

	-- Ethernet MAGJACK	LEDs
--	PHY_LED1 	<= NOSEQ; -- 
--	PHY_LED2 	<= GPIO_Output(0);   -- 

	-- Drive MAGJACK LEDS from FPGA I/O Pins
	PHY_LED1 <= Y2; -- keep NOSEQ as well for legacy reasons?
	PHY_LED2 <=	Y4;


	-- JTAG Chain multiplexer, best is to bypass directly
    C_TMS 		<= M_TMS;
    C_TCK 		<= M_TCK;
    C_TDI 		<= M_TDI;
	M_TDO 		<=	C_TDO; 
    --M_TDO 		<= C_TDO when FMC_JTAGEN='1' else FMC_TDO;

    FMC_TMS 		<= '0'; -- M_TMS when FMC_JTAGEN='1' else '0';
    FMC_TCK 		<= '0'; -- M_TCK when FMC_JTAGEN='1' else '0';
    FMC_TDI 		<= '0'; -- M_TDI when FMC_JTAGEN='1' else '0';
	
	ADBUS4 		<= '0'; -- M_TCK; -- not needed!


	-- Power/Reset Control

	-- FMC Related signals
	PG_C2M	 	<= FMC_POWERED;
	
	EN_FMC 		<= FMC_POWERED;
	FMC_POWERED <= (GPIO_output(7) and PON) when MODE3='1' else '1'; -- I2C in MODE3 or force on otherwise
	
	-- 000 = 3.3V 
	-- 001 = 2.5V
	-- 010 = 1.8V
	-- 011 = 1.5V
	--
	-- use dip switch M2, M1
	-- 00 1.8V
	-- 01 2.5V
	-- 10 3.3V
	-- 11 I2C setting
	--
	VID0  		<= GPIO_output(4) when MODE3='1' else '1' when MODE1='1' else '0';
	VID1  		<= GPIO_output(5) when MODE3='1' else '1' when MODE0='1' else '0';
	VID2  		<= GPIO_output(6) when MODE3='1' else '0';

--
-- XO2 Internal oscillator
--
MyInternalOSC: OSCH
		-- synthesis translate_off
		GENERIC MAP ( NOM_FREQ => "24.18" )
		-- synthesis translate_on
		PORT MAP ( 
			STDBY		=> '0',
			OSC			=> osc_clk,
			SEDSTDBY	=> open
		);

	GPIO_input(0) <= FMC_PRSNT;
	GPIO_input(1) <= POK_FMC;
    GPIO_input(2) <= SD_DETECT;
    GPIO_input(3) <= SD_WP;
	
    GPIO_input(4) <= USB_OC;
    GPIO_input(5) <= CM0;
    GPIO_input(6) <= CM1;
    GPIO_input(7) <= '0'; -- RFU
	
	

My_GPIO: I2C_to_GPIO8 port map (
		sda 	=> SDA,
        sclk 	=> SCL,
		
        GPIO_input =>  GPIO_input(7 downto 0),
        GPIO_output => GPIO_output(7 downto 0)
		);

Inst_PORESET: PORESET PORT MAP( 		
	SYSRST => ext_reset_n, 		
	RST => POR_DONE, 		
	CLK => Clk 	);

	process (POR_DONE, Clk)
	begin
		if POR_DONE = '0' then
			POR_DELAYED <= '0'; -- 
			por_cnt <= X"00000000";
		elsif Clk'event and Clk = '1' then
			por_cnt <= por_cnt + X"00000001";
			if por_cnt(23)='1' then
				POR_DELAYED <= '1';
			end if;
		end if;
	end process;


END ARCHITECTURE translated;